==> ezmlm-idx-0.40.tar.gz: ezmlm-idx-0.40/ezmlm-tstdig.c <==
/*$Id: ezmlm-tstdig.c,v 1.17 1999/03/20 16:43:42 lindberg Exp $*/
/*$Name: ezmlm-idx-040 $*/

#include <sys/types.h>
#include "stralloc.h"
#include "substdio.h"
#include "readwrite.h"
#include "strerr.h"
#include "sig.h"
#include "getconf.h"
#include "env.h"
#include "fmt.h"
#include "now.h"
#include "lock.h"
#include "sgetopt.h"
#include "errtxt.h"
#include "idx.h"

#define FATAL "ezmlm-tstdig: fatal: "

void die_usage()
{
  strerr_die1x(100,"ezmlm-tstdig: usage: ezmlm-tstdig [-k kbytes] [-m messages] [-t hours] dir");
}

void die_nomem() { strerr_die2x(111,FATAL,ERR_NOMEM); }

stralloc line = {0};

substdio ssnum;
char numbuf[16];

char strnum[FMT_ULONG];

int flaglocal = 0;

void main(argc,argv)
int argc;
char **argv;
{
  char *dir;
  char *local;
  char *def;
  int opt;
  unsigned int pos;
  unsigned long num, digsize, dignum;
  unsigned long cumsize = 0L;
  unsigned long deltanum = 0L;
  unsigned long deltawhen = 0L;
  unsigned long deltasize = 0L;
  unsigned long when, tsttime, digwhen;
  int fd,fdlock;

  (void) umask(022);
  sig_pipeignore();
  when = (unsigned long) now();

  while ((opt = getopt(argc,argv,"k:t:m:vV")) != opteof)
    switch(opt) {
      case 'k':
                if (optarg)
                  scan_ulong(optarg,&deltasize);
                break;
      case 't':
                if (optarg)	/* hours */
                  scan_ulong(optarg,&deltawhen);
                break;
      case 'm':
                if (optarg)
                  scan_ulong(optarg,&deltanum);
                break;
      case 'v':
      case 'V': strerr_die2x(0,"ezmlm-tstdig version: ",EZIDX_VERSION);
      default:
	die_usage();
    }


  dir = argv[optind++];
  if (!dir) die_usage();

  if (chdir(dir) == -1)
    strerr_die4sys(111,FATAL,ERR_SWITCH,dir,": ");

  if (argv[optind])
    die_usage();	/* avoid common error of putting options after dir */
  if (!getconf_line(&line,"num",0,FATAL,dir))
    _exit(99);		/* no msgs no shirt -> no digest */
  if(!stralloc_0(&line)) die_nomem();
  pos = scan_ulong(line.s,&num);
  if (line.s[pos] == ':')
    scan_ulong(line.s+pos+1,&cumsize);

  if (getconf_line(&line,"dignum",0,FATAL,dir)) {
    if(!stralloc_0(&line)) die_nomem();
    pos = scan_ulong(line.s,&dignum);
    if (line.s[pos] == ':')
      pos += 1 + scan_ulong(line.s+pos+1,&digsize);
    if (line.s[pos] == ':')
      scan_ulong(line.s+pos+1,&digwhen);
  } else {
    dignum = 0L;	/* no file, not done any digest */
    digsize = 0L;	/* nothing digested */
    digwhen = 0L;	/* will force a digest, but the last one was eons  */
			/* ago. ezmlm-get sends it out only if there are   */
			/* messages. This is as it should for new lists.   */
  }
  local = env_get("LOCAL");
  if (local && *local) {			/* in editor or manager */
    def = env_get("DEFAULT");
    if (def && *def) {				/* qmail>=1.02 and manager */
      if (!case_starts(def,"dig") || case_starts(def,"digest-"))
        _exit(0);
    } else {				/* older qmail versions or editor */
      if (!getconf_line(&line,"inlocal",0,FATAL,dir)) {
	flaglocal = 1;
      } else {
	pos = str_len(local);
	if (pos <= line.len) {		/* maybe qmail>=1.02 and editor */
	  flaglocal = 1;		/* editor and qmail>=1.02. No harm */
					/* if we're wrong */
	} else {			/* older qmail */
	  if (case_diffb(local,line.len,line.s))	/* local */
	    flaglocal = 1;		/* minimal harm */
          else if (pos < line.len +4 ||	/* in manager and non-digest */
		!case_starts(local+line.len,"-dig"))
	    _exit(0);
          else if (case_starts(local+line.len,"-digest-"))
	    _exit(0);
	}
      }
    }
  }

  if (!deltawhen && !deltasize && !deltanum) _exit(0);
  if ((deltawhen && ((digwhen + deltawhen * 3600L) <= when)) ||
      (deltasize && ((digsize + (deltasize << 2)) <= cumsize)) ||
      (deltanum && ((dignum + deltanum) <= num))) {	/* digest! */
    if (flaglocal) {	/* avoid multiple digests. Of course, ezmlm-tstdig*/
			/* belongs in ezmlm-digest, but it's too late ....*/
      fdlock = open_append("lock");
      if (fdlock == -1)
        strerr_die2sys(111,FATAL,ERR_OPEN_LOCK);
      if (lock_ex(fdlock) == -1) {
        close(fdlock);
        strerr_die2sys(111,FATAL,ERR_OBTAIN_LOCK);
      }
      getconf_line(&line,"tstdig",0,dir,FATAL);
      if (!stralloc_0(&line)) die_nomem();
      scan_ulong(line.s,&tsttime);	/* give digest 1 h to complete */
					/* nobody does digests more often */
      if ((tsttime + 3600L < when) || (tsttime <= digwhen)) {
        fd = open_trunc("tstdign");
        if (fd == -1)
          strerr_die6sys(111,FATAL,ERR_CREATE,dir,"/","tstdign",": ");
        substdio_fdbuf(&ssnum,write,fd,numbuf,sizeof(numbuf));
        if (substdio_put(&ssnum,strnum,fmt_ulong(strnum,when)) == -1)
          strerr_die6sys(111,FATAL,ERR_WRITE,dir,"/","tstdign",": ");
        if (substdio_puts(&ssnum,"\n") == -1)
          strerr_die6sys(111,FATAL,ERR_WRITE,dir,"/","tstdign",": ");
        if (substdio_flush(&ssnum) == -1)
          strerr_die6sys(111,FATAL,ERR_FLUSH,dir,"/","tstdign",": ");
        if (fsync(fd) == -1)
          strerr_die6sys(111,FATAL,ERR_SYNC,dir,"/","tstdign",": ");
        if (close(fd) == -1)
          strerr_die6sys(111,FATAL,ERR_CLOSE,dir,"/","tstdign",": ");
        if (rename("tstdign","tstdig") == -1)
          strerr_die4sys(111,FATAL,ERR_MOVE,"tstdign",": ");
        _exit(0);
      }
    } else
        _exit(0);
  }
  _exit(99);
}

==> ezmlm-idx_7.1.1.orig.tar.gz: ezmlm-idx-7.1.1/ezmlm-tstdig.c <==
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <unistd.h>
#include "stralloc.h"
#include "substdio.h"
#include "readwrite.h"
#include "strerr.h"
#include "sig.h"
#include "getconf.h"
#include "env.h"
#include "fmt.h"
#include "now.h"
#include "lock.h"
#include "getconfopt.h"
#include "messages.h"
#include "scan.h"
#include "case.h"
#include "str.h"
#include "open.h"
#include "die.h"
#include "wrap.h"
#include "idx.h"
#include "config.h"

const char FATAL[] = "ezmlm-tstdig: fatal: ";
const char USAGE[] =
"ezmlm-tstdig: usage: ezmlm-tstdig [-k kbytes] [-m messages] [-t hours] dir";

static unsigned long deltanum = ~0UL;
static unsigned long deltawhen = ~0UL;
static unsigned long deltasize = ~0UL;
static struct option options[] = {
  OPT_ULONG(deltasize,'k',"digsize"),
  OPT_ULONG(deltanum,'m',"digcount"),
  OPT_ULONG(deltawhen,'t',"digtime"),
  OPT_END
};

stralloc line = {0};

substdio ssnum;
char numbuf[16];

char strnum[FMT_ULONG];

int flaglocal = 0;

int main(int argc,char **argv)
{
  char *local;
  char *def;
  unsigned int pos;
  unsigned long num, digsize, dignum;
  unsigned long cumsize = 0L;
  unsigned long when, tsttime, digwhen;
  int fd,fdlock;

  (void) umask(022);
  sig_pipeignore();
  when = (unsigned long) now();
  optind = getconfopt(argc,argv,options,1,0);
  if (argv[optind])
    die_usage();	/* avoid common error of putting options after dir */
  if (!getconf_ulong2(&num,&cumsize,"num",0))
    _exit(99);

  if (getconf_line(&line,"dignum",0)) {
    if(!stralloc_0(&line)) die_nomem();
    pos = scan_ulong(line.s,&dignum);
    if (line.s[pos] == ':')
      pos += 1 + scan_ulong(line.s+pos+1,&digsize);
    if (line.s[pos] == ':')
      scan_ulong(line.s+pos+1,&digwhen);
  } else {
    dignum = 0L;	/* no file, not done any digest */
    digsize = 0L;	/* nothing digested */
    digwhen = 0L;	/* will force a digest, but the last one was eons  */
			/* ago. ezmlm-get sends it out only if there are   */
			/* messages. This is as it should for new lists.   */
  }
  local = env_get("LOCAL");
  if (local && *local) {			/* in editor or manager */
    def = env_get("DEFAULT");
    if (def && *def) {				/* in manager */
      if (!case_starts(def,"dig") || case_starts(def,"digest-"))
        _exit(0);
    } else {					/* in editor */
      flaglocal = 1;
    }
  }

  if (deltawhen == ~0UL)
    deltawhen = 48;
  if (deltasize == ~0UL)
    deltasize = 64;
  if (deltanum == ~0UL)
    deltanum = 30;
  if ((deltawhen && ((digwhen + deltawhen * 3600L) <= when)) ||
      (deltasize && ((digsize + (deltasize << 2)) <= cumsize)) ||
      (deltanum && ((dignum + deltanum) <= num))) {	/* digest! */
    if (flaglocal) {	/* avoid multiple digests. Of course, ezmlm-tstdig*/
			/* belongs in ezmlm-digest, but it's too late ....*/
      fdlock = lockfile("lock");
      getconf_line(&line,"tstdig",0);
      if (!stralloc_0(&line)) die_nomem();
      scan_ulong(line.s,&tsttime);	/* give digest 1 h to complete */
					/* nobody does digests more often */
      if ((tsttime + 3600L < when) || (tsttime <= digwhen)) {
        fd = open_trunc("tstdign");
        if (fd == -1)
          strerr_die2sys(111,FATAL,MSG1(ERR_CREATE,"tstdign"));
        substdio_fdbuf(&ssnum,write,fd,numbuf,sizeof(numbuf));
        if (substdio_put(&ssnum,strnum,fmt_ulong(strnum,when)) == -1)
          strerr_die2sys(111,FATAL,MSG1(ERR_WRITE,"tstdign"));
        if (substdio_puts(&ssnum,"\n") == -1)
          strerr_die2sys(111,FATAL,MSG1(ERR_WRITE,"tstdign"));
        if (substdio_flush(&ssnum) == -1)
          strerr_die2sys(111,FATAL,MSG1(ERR_FLUSH,"tstdign"));
        if (fsync(fd) == -1)
          strerr_die2sys(111,FATAL,MSG1(ERR_SYNC,"tstdign"));
        if (close(fd) == -1)
          strerr_die2sys(111,FATAL,MSG1(ERR_CLOSE,"tstdign"));
        wrap_rename("tstdign","tstdig");
        _exit(0);
      }
    } else
        _exit(0);
  }
  _exit(99);
}
